---------------------------------------------------------------
-- Universidade Federal de Pernambuco - UFPE
-- Centro de Informatica - CIn
-- Projeto: HPCIn
-- Coordenador: Manoel Eusebio
----------------------------------------------------------------
----------------------------------------------------------------
-- Modulo: leitor_memoria.vhd
-- Descricao: 	Modulo responsavel por ler a memoria de saida
--				do multiplicador matriz vetor e enviar os 
--				valores para os displays de 7 segmentos da DE2
-- 
-- Autor(es): 	Victor Medeiros <victorwcm@gmail.com>
--				Viviane Lucy <vivianelucy@gmail.com>
--
-- Data de criacao: 29/07/08
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- definicao da entidade
entity memory_reader is port (
	Prst 				: in std_logic;
	Pclk				: in std_logic;
	-- dado lido da memoria
	Pdata_in			: in std_logic_vector(31 downto 0);
	-- endereco a ser lido da memoria
	Prd_addr			: out std_logic_vector(3 downto 0);
	-- saidas para os displays de 7 segmentos
	Pdisplay0			: out std_logic_vector(6 downto 0);
	Pdisplay1			: out std_logic_vector(6 downto 0);
	Pdisplay2			: out std_logic_vector(6 downto 0);
	Pdisplay3			: out std_logic_vector(6 downto 0);
	-- entrada que dispara o inicio da leitura do conteudo da memoria
	Prd_memory_button	: in  std_logic;
	-- sinaliza o final da leitura dos dados atraves de um LED
	Pend_led			: out std_logic
);
end memory_reader;

-- descricao da arquitetura da entidade
architecture arq_memory_reader of memory_reader is
	--definicao dos sinais
	-- sinal que armazena a posicao que esta sendo lida atualmente
	signal Smemory_index : std_logic_vector(3 downto 0) := "0000";

	-- definicao do decodificador BCD utilizado no modulo
	component bcd_decoder
		port(
			Phex	: in std_logic_vector(3 downto 0);
			Pled	: out std_logic_vector(6 downto 0)
		);
	end component;

	begin
	
	-- instancia de um decodificador BCD para o display 0
	display0 : bcd_decoder port map (
		Phex => Pdata_in(3 downto 0),
		Pled => Pdisplay0
	);
	
	-- instancia de um decodificador BCD para o display 1
	display1 : bcd_decoder port map (
		Phex => Pdata_in(7 downto 4),
		Pled => Pdisplay1
	);
	
	-- instancia de um decodificador BCD para o display 2
	display2 : bcd_decoder port map (
		Phex => Pdata_in(11 downto 8),
		Pled => Pdisplay2
	);
	
	-- instancia de um decodificador BCD para o display 3
	display3 : bcd_decoder port map (
		Phex => Pdata_in(15 downto 12),
		Pled => Pdisplay3
	);
	
	-- processo que implementa o comportamento do modulo
	process (Pclk, Prst, Prd_memory_button)
		begin
			-- sincroniza com o clock 
			if(Pclk'event and Pclk = '1') then
				-- se o reset foi apertado
				if (Prst = '0') then 
					Smemory_index <= (others => '0');
					Pend_led <= '1';	
				-- caso tenha sido apertado o botao para disparar a leitura
				elsif (Prd_memory_button = '1') then
					Smemory_index <= Smemory_index + 1;
					Pend_led <= '0';
				end if;
				-- caso tenha atigido a ultima posicao da memoria volta a primeira
				if Smemory_index = "1010" then
					Smemory_index <= (others => '0');
					Pend_led <= '1';
				end if;
			end if;
	end process;
	-- atualiza o endereco atual na porta
	Prd_addr <= Smemory_index;
end arq_memory_reader;