---------------------------------------------------------------
-- Universidade Federal de Pernambuco - UFPE
-- Centro de Informatica - CIn
-- Projeto: HPCIn
-- Coordenador: Manoel Eusebio
----------------------------------------------------------------
----------------------------------------------------------------
-- Modulo: decodificador_bcd.vhd
-- Descricao: 	Modulo responsavel por decodificar um valor 
-- 				hexadecimal para um display de 7 segmentos
-- 				Distribuicao dos bits nos segmentos do display
--					 0
--					---  
--				 5 |   | 1
--					---   <- 6
--				 4 |   | 2
--					---
--					 3
-- 
-- Autor(es): 	Victor Medeiros <victorwcm@gmail.com>
--				Viviane Lucy <vivianelucy@gmail.com>
--
-- Data de criacao: 29/07/08
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- definicao da entidade
entity bcd_decoder is 
	port (
		-- dado de entrada a ser convertido
		Phex	: in std_logic_vector(3 downto 0);
		-- saida para o display de 7 segmentos
		Pled	: out std_logic_vector(6 downto 0)
	);
end bcd_decoder;

-- descricao da arquitetura da entidade
architecture arq_bcd_decoder of bcd_decoder is
	begin
		with Phex select
			Pled <= "1111001" when "0001",   --1
					"0100100" when "0010",   --2
					"0110000" when "0011",   --3
					"0011001" when "0100",   --4
					"0010010" when "0101",   --5
					"0000010" when "0110",   --6
					"1111000" when "0111",   --7
					"0000000" when "1000",   --8
					"0010000" when "1001",   --9
					"0001000" when "1010",   --A
					"0000011" when "1011",   --b
					"1000110" when "1100",   --C
					"0100001" when "1101",   --d
					"0000110" when "1110",   --E
					"0001110" when "1111",   --F
					"1000000" when others;   --0
end arq_bcd_decoder;
  