---------------------------------------------------------------
-- Universidade Federal de Pernambuco - UFPE
-- Centro de Informatica - CIn
-- Projeto: HPCIn
-- Coordenador: Manoel Eusebio
----------------------------------------------------------------
----------------------------------------------------------------
-- Modulo: debounce.vhd
-- Descricao: 	Modulo responsavel por realizar o debounce de
--				botoes.
-- 
-- Autor(es): 	Victor Medeiros <victorwcm@gmail.com>
--				Viviane Lucy <vivianelucy@gmail.com>
--
-- Data de criacao: 29/07/08
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- definicao da entidade
entity debounce is port (
	Prst 		: in std_logic;
	Pclk		: in std_logic;
	Pin		 	: in std_logic;
	Pout		: out std_logic
);
end debounce;

-- definicao da arquitetura da entidade
architecture arq_debounce of debounce is 
	signal Q1, Q2, Q3 	: std_logic;	
	begin
		process (Pclk, Prst)
			begin
				if (Pclk'event and Pclk = '1') then 
					if (Prst = '0') then
						Q1 <= '0';
						Q2 <= '0';
						Q3 <= '0';  
					else 
						Q1 <= not Pin;
						Q2 <= Q1;
						Q3 <= Q2;
					end if;
				end if;
		end process;	
	Pout <= (Q1 and Q2 and (not Q3));
end arq_debounce;