---------------------------------------------------------------
-- Universidade Federal de Pernambuco - UFPE
-- Centro de Informatica - CIn
-- Projeto: HPCIn
-- Coordenador: Manoel Eusebio
----------------------------------------------------------------
----------------------------------------------------------------
-- Modulo: mult_mat_vector.vhd
-- Descricao: 	Modulo responsavel por implementar o 
--				multiplicador-acumulador de 32 bits
-- 
-- Autor(es): 	Viviane Lucy <vivianelucy@gmail.com>
--				Victor Medeiros <victorwcm@gmail.com>
--
-- Data de criacao: 21/07/08
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- definicao da entidade
entity mac32bits is 
	port (
		Prst 		: in std_logic;
		Pclk		: in std_logic;
		Pmult_vld 	: in std_logic;
		Pdone_ac	: in std_logic;
		Pinput_a	: in std_logic_vector(31 downto 0);
		Pinput_b	: in std_logic_vector(31 downto 0);
		Poutput		: out std_logic_vector(31 downto 0)
	);
end mac32bits;

-- descricao da arquitetura comportamental da entidade
architecture arq_mac32bits of mac32bits is
	-- definicao do registrador acumulador
	signal Saccumulator : std_logic_vector (31 downto 0);
	begin
		process (Pclk, Prst, Pmult_vld, Pdone_ac)
			-- declaracao das variaveis do processo
			-- variavel temporaria
			variable Vtemp_mult : std_logic_vector (63 downto 0);	
			begin
				-- sincronismo com o clock
				if (Pclk'event and Pclk = '1') then
					-- se o reset foi pressionado
					if (Prst = '0') then 
						Vtemp_mult := (others => '0');
						Saccumulator <= (others => '0');
						Poutput <= x"FFFFFFFF";
					
					else
						if (Pdone_ac = '0' and Pmult_vld = '1') then
							Vtemp_mult := Pinput_a * Pinput_b;
							Saccumulator <= Saccumulator + Vtemp_mult (31 downto 0);
						elsif (Pdone_ac = '1') then 
							Poutput <= Saccumulator (31 downto 0);
							Saccumulator <= (others => '0');
						end if;
					end if;
				end if;
		end process;
end arq_mac32bits;