---------------------------------------------------------------
-- Universidade Federal de Pernambuco - UFPE
-- Centro de Informatica - CIn
-- Projeto: HPCIn
-- Coordenador: Manoel Eusebio
----------------------------------------------------------------
----------------------------------------------------------------
-- Modulo: mult_mat_vector.vhd
-- Descricao: Top do multiplicador matriz vetor
-- 
-- Autor(es): 	Viviane Lucy <vivianelucy@gmail.com>
--				Victor Medeiros <victorwcm@gmail.com>
--
-- Data de criacao: 21/07/08
----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use IEEE.std_logic_arith.all;

-- definicao da entidade
entity mult_mat_vector is
	port(
		-- portas gerais
		Pclk				: in std_logic;
		Prst				: in std_logic;		
		-- portas para conexao com a placa DE2
		Pinit_button		: in std_logic;
		Prd_memory_button	: in std_logic;
		Pend_led			: out std_logic;
		Pdisplay0			: out std_logic_vector(6 downto 0);
		Pdisplay1			: out std_logic_vector(6 downto 0); 
		Pdisplay2			: out std_logic_vector(6 downto 0); 
		Pdisplay3			: out std_logic_vector(6 downto 0) 
	);
end mult_mat_vector;

-- descricao da arquitetura comportamental da entidade 
architecture arch_mult_mat_vector of mult_mat_vector is
	-- definicao dos sinais
	-- sinais de controle do multiplicador
	signal Sdone				: std_logic;
	signal Smac_vld				: std_logic;
	signal Sinit_button 		: std_logic;
	signal Srd_memory_button 	: std_logic;
	
	-- sinais relacionados as memorias que armazenam a matriz
	-- o mesmo sinal de enderecamento servira para todas as memorias
	signal Saddr_mem_matrix			: std_logic_vector(6 downto 0);
	
	signal Sdata_out_mem_matrix_1	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_1	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_1			: std_logic;
	
	signal Sdata_out_mem_matrix_2	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_2	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_2			: std_logic;
	
	signal Sdata_out_mem_matrix_3	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_3	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_3			: std_logic;
	
	signal Sdata_out_mem_matrix_4	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_4	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_4			: std_logic;
	
	signal Sdata_out_mem_matrix_5	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_5	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_5			: std_logic;
	
	signal Sdata_out_mem_matrix_6	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_6	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_6			: std_logic;
	
	signal Sdata_out_mem_matrix_7	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_7	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_7			: std_logic;
	
	signal Sdata_out_mem_matrix_8	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_8	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_8			: std_logic;
	
	signal Sdata_out_mem_matrix_9	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_9	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_9			: std_logic;
	
	signal Sdata_out_mem_matrix_10	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_matrix_10	: std_logic_vector(31 downto 0);
	signal Swr_mem_matrix_10		: std_logic;
	
	-- sinais relacionados a memoria que armazena o vetor
	signal Saddr_mem_vector		: std_logic_vector(6 downto 0);
	signal Sdata_out_mem_vector	: std_logic_vector(31 downto 0);
	signal Sdata_in_mem_vector	: std_logic_vector(31 downto 0);
	signal Swr_mem_vector		: std_logic;
	
	-- sinais relacionados ao mux que chaveia o MAC a ser lido
	signal Smac_sel			: std_logic_vector(3 downto 0);
	signal Sdata_in_mux_1 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_2 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_3 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_4 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_5 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_6 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_7 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_8 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_9 	: std_logic_vector(31 downto 0);
	signal Sdata_in_mux_10 	: std_logic_vector(31 downto 0);
	signal Sdata_out_mux	: std_logic_vector(31 downto 0);

	-- definicao dos componentes
	-- definicao do controlador
	component mem_control
		port (
			Pclk				: 	in std_logic;
			Prst				: 	in std_logic;
		
			-- portas de conexao com as memorias
			-- porta que endereca a memoria que contem a matriz
			Paddr_mem_matrix	: out std_logic_vector(6 downto 0);
			-- porta que endereca a memoria que contem o vetor
			Paddr_mem_vector	: out std_logic_vector(6 downto 0);
		
			-- sinais de controle
			-- porta que indica que o botao de init foi pressionado
			Pinit_button		: in std_logic;
			-- porta que indica para o MAC que ja existem dados validos
			Pmac_vld: out std_logic;
			-- sinaliza para o MAC que todos os produtos ja foram realizados
			Pdone: out std_logic
		);
	end component;
	
	-- definicao da memoria que armezena a linha 1 da matriz
	component memory_matrix_line_1
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 2 da matriz
	component memory_matrix_line_2
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 3 da matriz
	component memory_matrix_line_3
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 4 da matriz
	component memory_matrix_line_4
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 5 da matriz
	component memory_matrix_line_5
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 6 da matriz
	component memory_matrix_line_6
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 7 da matriz
	component memory_matrix_line_7
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 8 da matriz
	component memory_matrix_line_8
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 9 da matriz
	component memory_matrix_line_9
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao da memoria que armezena a linha 10 da matriz
	component memory_matrix_line_10
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;

	-- definicao da memoria que armazena o vetor
	component memory_vector
		port (
			address	: in std_logic_vector(6 downto 0);
			clock	: in std_logic;
			data	: in std_logic_vector(31 downto 0);
			wren	: in std_logic;
			q		: out std_logic_vector(31 downto 0)
		);
	end component;
	
	-- definicao do multiplicador acumulador de 32 bits
	component mac32bits 
		port (
			Prst 		: in std_logic;
			Pclk		: in std_logic;
			Pmult_vld 	: in std_logic;
			Pdone_ac	: in std_logic;
			Pinput_a	: in std_logic_vector (31 downto 0);
			Pinput_b	: in std_logic_vector (31 downto 0);
			Poutput		: out std_logic_vector (31 downto 0)
		);
	end component;

	-- definicao do componente responsavel por ler a memoria de saida e exibir os valores
	-- nos displays de 7 segmentos da DE2
	component memory_reader
		port (
			Prst 				: in std_logic;
			Pclk				: in std_logic;
			-- dado lido da memoria
			Pdata_in			: in std_logic_vector(31 downto 0);
			-- seleciona no mux o MAC a ser lido
			Pmac_sel			: out std_logic_vector(3 downto 0);
			-- saidas para os displays de 7 segmentos
			Pdisplay0			: out std_logic_vector(6 downto 0);
			Pdisplay1			: out std_logic_vector(6 downto 0);
			Pdisplay2			: out std_logic_vector(6 downto 0);
			Pdisplay3			: out std_logic_vector(6 downto 0);
			-- entrada que dispara o inicio da leitura do conteudo da memoria
			Prd_memory_button	: in  std_logic;
			-- sinaliza o final da leitura dos dados atraves de um LED
			Pend_led			: out std_logic
		);
	end component;

	-- definicao do componente responsavel por realizar o debounce dos botoes
	component debounce
		port (
			Prst 	: in std_logic;
			Pclk	: in std_logic;
			Pin		: in std_logic;
			Pout	: out std_logic
		);
	end component;
	
	-- definicao do mux que chaveia a escrita dos MACs na memoria de saida
	component mux_10_1
		port (
			Psel		: in std_logic_vector(3 downto 0);
			Pinput_1	: in std_logic_vector(31 downto 0);
			Pinput_2	: in std_logic_vector(31 downto 0);
			Pinput_3	: in std_logic_vector(31 downto 0);
			Pinput_4	: in std_logic_vector(31 downto 0);
			Pinput_5	: in std_logic_vector(31 downto 0);
			Pinput_6	: in std_logic_vector(31 downto 0);
			Pinput_7	: in std_logic_vector(31 downto 0);
			Pinput_8	: in std_logic_vector(31 downto 0);
			Pinput_9	: in std_logic_vector(31 downto 0);
			Pinput_10	: in std_logic_vector(31 downto 0);
			Poutput		: out std_logic_vector(31 downto 0)
		);
	end component;

	begin

	Sdata_in_mem_matrix_1 	<= x"00000000";
	Swr_mem_matrix_1 		<= '0';
	Sdata_in_mem_matrix_2 	<= x"00000000";
	Swr_mem_matrix_2		<= '0';
	Sdata_in_mem_matrix_3 	<= x"00000000";
	Swr_mem_matrix_3	 	<= '0';
	Sdata_in_mem_matrix_4 	<= x"00000000";
	Swr_mem_matrix_4		<= '0';
	Sdata_in_mem_matrix_5 	<= x"00000000";
	Swr_mem_matrix_5 		<= '0';
	Sdata_in_mem_matrix_6 	<= x"00000000";
	Swr_mem_matrix_6		<= '0';
	Sdata_in_mem_matrix_7 	<= x"00000000";
	Swr_mem_matrix_7 		<= '0';
	Sdata_in_mem_matrix_8 	<= x"00000000";
	Swr_mem_matrix_8		<= '0';
	Sdata_in_mem_matrix_9 	<= x"00000000";
	Swr_mem_matrix_9 		<= '0';
	Sdata_in_mem_matrix_10 	<= x"00000000";
	Swr_mem_matrix_10		<= '0';
	Sdata_in_mem_vector		<= x"00000000";
	Swr_mem_vector			<= '0';

	-- instancia dos componentes
	-- instancia do modulo controlador
	mem_ctrl : mem_control
        port map (
            Pclk => Pclk,
			Prst => Prst,
			Paddr_mem_matrix 	=> Saddr_mem_matrix,
			Paddr_mem_vector 	=> Saddr_mem_vector,
			Pinit_button 		=> Sinit_button,
			Pmac_vld 			=> Smac_vld,
			Pdone 				=> Sdone
		);
	
	-- instancia da memoria que armazena a linha 1 da matriz 
	mem_matrix_1 : memory_matrix_line_1
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_1,
			wren	=> Swr_mem_matrix_1,
			q	 	=> Sdata_out_mem_matrix_1
		);
		
	-- instancia da memoria que armazena a linha 2 da matriz 
	mem_matrix_2 : memory_matrix_line_2
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_2,
			wren	=> Swr_mem_matrix_2,
			q	 	=> Sdata_out_mem_matrix_2
		);
	-- instancia da memoria que armazena a linha 3 da matriz 
	mem_matrix_3 : memory_matrix_line_3
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_3,
			wren	=> Swr_mem_matrix_3,
			q	 	=> Sdata_out_mem_matrix_3
		);
		
	-- instancia da memoria que armazena a linha 4 da matriz 
	mem_matrix_4 : memory_matrix_line_4
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_4,
			wren	=> Swr_mem_matrix_4,
			q	 	=> Sdata_out_mem_matrix_4
		);
		
	-- instancia da memoria que armazena a linha 5 da matriz 
	mem_matrix_5 : memory_matrix_line_5
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_5,
			wren	=> Swr_mem_matrix_5,
			q	 	=> Sdata_out_mem_matrix_5
		);
		
	-- instancia da memoria que armazena a linha 6 da matriz 
	mem_matrix_6 : memory_matrix_line_6
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_6,
			wren	=> Swr_mem_matrix_6,
			q	 	=> Sdata_out_mem_matrix_6
		);
		
	-- instancia da memoria que armazena a linha 7 da matriz 
	mem_matrix_7 : memory_matrix_line_7
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_7,
			wren	=> Swr_mem_matrix_7,
			q	 	=> Sdata_out_mem_matrix_7
		);
		
	-- instancia da memoria que armazena a linha 8 da matriz 
	mem_matrix_8 : memory_matrix_line_8
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_8,
			wren	=> Swr_mem_matrix_8,
			q	 	=> Sdata_out_mem_matrix_8
		);
		
	-- instancia da memoria que armazena a linha 9 da matriz 
	mem_matrix_9 : memory_matrix_line_9
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_9,
			wren	=> Swr_mem_matrix_9,
			q	 	=> Sdata_out_mem_matrix_9
		);

	-- instancia da memoria que armazena a linha 10 da matriz 
	mem_matrix_10 : memory_matrix_line_10
		port map (
			address	=> Saddr_mem_matrix,
			clock	=> Pclk,
			data	=> Sdata_in_mem_matrix_10,
			wren	=> Swr_mem_matrix_10,
			q	 	=> Sdata_out_mem_matrix_10
		);
	
	-- instancia da memoria que armazena o vetor			
	mem_vector : memory_vector
		port map (
			address	=> Saddr_mem_vector,
			clock	=> Pclk,
			data	=> Sdata_in_mem_vector,
			wren	=> Swr_mem_vector,
			q	 	=> Sdata_out_mem_vector
		);
	
	-- instancia do MAC 1
	mac_1 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_1,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_1
		);

	-- instancia do MAC 2
	mac_2 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_2,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_2
		);

	-- instancia do MAC 3
	mac_3 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_3,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_3
		);

	-- instancia do MAC 4
	mac_4 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_4,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_4
		);

	-- instancia do MAC 5
	mac_5 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_5,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_5
		);

	-- instancia do MAC 6
	mac_6 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_6,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_6
		);

	-- instancia do MAC 7
	mac_7 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_7,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_7
		);

	-- instancia do MAC 8
	mac_8 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_8,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_8
		);

	-- instancia do MAC 9
	mac_9 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_9,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_9
		);

	-- instancia do MAC 10
	mac_10 : mac32bits 
		port map (
			Prst 		=> Prst,
			Pclk 		=> Pclk,
			Pmult_vld 	=> Smac_vld,
			Pdone_ac 	=> Sdone,
			Pinput_a 	=> Sdata_out_mem_matrix_10,
			Pinput_b	=> Sdata_out_mem_vector,
			Poutput	 	=> Sdata_in_mux_10
		);

	--  instancia do modulo leitor de resultados da memoria de saida
	reader : memory_reader 
		port map (
			Prst 		=> Prst,
			Pclk		=> Pclk,
			Pdata_in	=> Sdata_out_mux,
			Pmac_sel	=> Smac_sel,
			-- saidas para os displays de 7 segmentos
			Pdisplay0	=> Pdisplay0,
			Pdisplay1	=> Pdisplay1,
			Pdisplay2	=> Pdisplay2,
			Pdisplay3	=> Pdisplay3, 
			-- entrada que dispara o inicio da leitura do conteudo da memoria
			Prd_memory_button => Srd_memory_button,
			-- sinaliza o final da leitura atraves do LED
			Pend_led	=> Pend_led
		);

	-- instancia do debounce para o botao init
	debounce_init : debounce 
		port map (
			Prst	=> Prst,
			Pclk	=> Pclk,
			Pin		=> Pinit_button,
			Pout	=> Sinit_button
		);

	-- instancia do debounce para o botao de leitura da memoria
	debounce_memory_reader : debounce 
		port map (
			Prst	=> Prst,
			Pclk	=> Pclk,
			Pin 	=> Prd_memory_button,
			Pout	=> Srd_memory_button
		);
	
	mux : mux_10_1
		port map (
			Psel		=> Smac_sel,
			Pinput_1	=> Sdata_in_mux_1,
			Pinput_2	=> Sdata_in_mux_2,
			Pinput_3	=> Sdata_in_mux_3,
			Pinput_4	=> Sdata_in_mux_4,
			Pinput_5	=> Sdata_in_mux_5,
			Pinput_6	=> Sdata_in_mux_6,
			Pinput_7	=> Sdata_in_mux_7,
			Pinput_8	=> Sdata_in_mux_8,
			Pinput_9	=> Sdata_in_mux_9,
			Pinput_10	=> Sdata_in_mux_10,
			Poutput		=> Sdata_out_mux
		);
end arch_mult_mat_vector;